"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const classnames_1 = require("classnames");
const react_pose_1 = require("react-pose");
const { useEffect, useRef } = React;
const DialogContainer = react_pose_1.default.div({
    preEnter: {
        opacity: 0
    },
    enter: {
        opacity: 1
    },
    exit: {
        opacity: 0,
        delay: 300,
        transition: {
            ease: 'easeIn',
            duration: 250
        }
    }
});
const DialogWrapper = react_pose_1.default.div({
    preEnter: {
        opacity: 0,
        y: '-60%'
    },
    enter: {
        opacity: 1,
        y: '-5%',
        delay: 250,
        transition: {
            duration: 350
        }
    },
    exit: {
        opacity: 0,
        y: '-60%',
        transition: {
            ease: 'easeIn'
        }
    }
});
exports.Dialog = React.memo(({ children, className, color, onClose, opacity, open, style }) => {
    // Hooks:
    const background = useRef(null);
    const content = useRef(null);
    useEffect(() => {
        content.current && content.current.focus();
    });
    return (React.createElement(react_pose_1.PoseGroup, { preEnterPose: "preEnter" }, open && (React.createElement(DialogContainer, { className: classnames_1.default(`Dialog fixed pin overflow-y-auto overflow-x-hidden w-full h-screen flex justify-center items-center`), key: "DialogContainer" },
        React.createElement("div", { className: classnames_1.default(`Dialog-background pin bg-${color} overflow-y-auto overflow-x-hidden fixed w-full h-screen opacity-${opacity}`), onClick: (e) => e.target === background.current && onClose(), ref: background }),
        React.createElement(DialogWrapper, { className: classnames_1.default('Dialog-content bg-white min-w-6 p-4 focus:outline-none overflow-y-auto', className && className), onKeyDown: (e) => e.keyCode === 27 && onClose(), style: style, tabIndex: "0", ref: content }, children)))));
});
exports.Dialog.defaultProps = {
    color: 'black',
    opacity: 50
};
