"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Components:
const React = require("react");
const classnames_1 = require("classnames");
const react_pose_1 = require("react-pose");
const createState_1 = require("../../utils/createState");
// ================================================================================================
const keyCode = {
    ESCAPE: 27,
    ENTER: 13,
    UP: 38,
    DOWN: 40
};
const AnimatedList = react_pose_1.default.div({
    preEnter: {
        opacity: 0
    },
    enter: {
        delayChildren: 100,
        opacity: 1,
        staggerChildren: 50
    },
    exit: {
        opacity: 0,
        transition: {
            ease: 'easeIn',
            duration: 250
        }
    }
});
const AnimatedListItem = react_pose_1.default.div({
    preEnter: {
        opacity: 0,
        y: '-20%'
    },
    enter: {
        opacity: 1,
        y: 0
    },
    exit: {
        opacity: 0
    }
});
exports.Selectbox = React.memo(props => {
    const { arrow, className, id, label, material, multiple, multipleSave, native, name, onSelect, options, selection, style } = props;
    let value = null;
    // @ts-ignore
    if (multiple && selection) {
        // @ts-ignore
        const selected = options.filter(option => selection.includes(option.value));
        value = selected.length === 0 ? null : selected.map(sel => sel.label).join(', ');
    }
    else {
        const selected = options.find(option => option.value === selection);
        value = selected ? selected.label : null;
    }
    // Hooks:
    const [state, setState] = createState_1.createState({
        open: false,
        selIndex: -1
    });
    const { open, selIndex } = state;
    const selectContainer = React.useRef(null);
    const listContainer = React.useRef(null);
    React.useEffect(() => {
        document.addEventListener('mousedown', handleClickOutSide);
        return () => {
            document.removeEventListener('mousedown', handleClickOutSide);
        };
    });
    // Handlers:
    const toogleOpen = () => {
        setState({ open: !open, selIndex: -1 });
    };
    const handleClickOutSide = (event) => {
        try {
            // @ts-ignore
            !(event.target.className.indexOf('Selectbox') > -1) && open && toogleOpen();
        }
        catch (error) {
            open && toogleOpen();
        }
    };
    const handleSelect = (option) => {
        if (multiple) {
            if (selection) {
                // @ts-ignore
                const selected = [...selection];
                const index = selected.findIndex(sel => option.value === sel);
                index > -1 ? selected.splice(index, 1) : selected.push(option.value);
                // @ts-ignore
                onSelect(selected);
            }
            else {
                onSelect([option.value]);
            }
        }
        else {
            onSelect(option.value);
            toogleOpen();
        }
    };
    const handleKeyDown = (e) => {
        switch (e.keyCode) {
            case keyCode.ESCAPE:
                setState({ open: false, selIndex: -1 });
                e.stopPropagation();
                break;
            case keyCode.ENTER:
                multiple && open && selIndex > -1 ? handleSelect(options[selIndex]) : toogleOpen();
                break;
            case keyCode.DOWN:
            case keyCode.UP:
                open ? handleChangeSelect(e.keyCode) : setState({ open: true });
                break;
            default:
                break;
        }
        for (let key in keyCode) {
            if (keyCode[key] === e.keyCode && selectContainer) {
                selectContainer.current.focus();
                e.preventDefault();
            }
        }
    };
    const handleChangeSelect = (key) => {
        let curIndex = 0;
        if (multiple) {
            if (selIndex > -1) {
                curIndex = selIndex;
            }
            else {
                setState({ selIndex: 0 });
                return;
            }
        }
        else {
            curIndex = options.findIndex(option => option.value === selection);
        }
        if (key === keyCode.DOWN) {
            curIndex < options.length - 1 ? curIndex++ : curIndex;
        }
        else {
            curIndex > 0 ? curIndex-- : curIndex;
        }
        multiple ? setState({ selIndex: curIndex }) : onSelect(options[curIndex].value);
        handleListScroll(curIndex);
    };
    const handleListScroll = (index) => {
        if (listContainer.current) {
            const { top: listTop, bottom: listBottom } = listContainer.current.getBoundingClientRect();
            const { top: rowTop, bottom: rowBottom } = listContainer.current.childNodes[index].getBoundingClientRect();
            // Check row inside list container view
            if (rowTop < listTop || rowBottom > listBottom) {
                listContainer.current.scroll({
                    top: listContainer.current.childNodes[index].offsetTop
                });
            }
        }
    };
    // Render:
    if (native && !multiple) {
        return (React.createElement("div", { className: classnames_1.default(`Selectbox native relative w-full  hover:cursor-pointer `, material && 'material', open && 'focus', className && className), style: style },
            arrow ? (arrow) : (React.createElement("div", { className: classnames_1.default('Selectbox-arrow absolute w-3 h-3 border-black  border-b-2 border-r-2') })),
            React.createElement("label", { className: classnames_1.default(`Selectbox-label absolute transition`, selection ? 'text-xs populated' : 'text-base'), htmlFor: id }, label),
            React.createElement("select", { className: classnames_1.default('Selectbox-select w-full focus:outline-none'), id: id, name: name, 
                // @ts-ignore
                value: selection, onFocus: toogleOpen, onBlur: toogleOpen, onChange: (event) => onSelect(event.target.value) },
                React.createElement("option", null),
                options.map(option => (React.createElement("option", { key: option.value, value: option.value }, option.label))))));
    }
    return (React.createElement("div", { className: classnames_1.default(`Selectbox relative w-full overflow-y-visible hover:cursor-pointer focus:outline-none transition`, material && 'material', open && 'focus', className && className), id: id, tabIndex: 0, ref: selectContainer, style: style, onKeyDown: handleKeyDown },
        React.createElement("button", { "aria-haspopup": "listbox", "aria-expanded": open, className: classnames_1.default(`Selectbox-trigger relative flex content-between items-center w-full focus:outline-none`), onClick: toogleOpen },
            React.createElement("label", { className: classnames_1.default(`Selectbox-label absolute pin-left transition`, value || open ? 'text-xs populated' : 'text-base') }, label),
            React.createElement("div", { className: classnames_1.default(`Selectbox-value relative pt-2 w-full text-left truncate`) },
                React.createElement("span", { className: classnames_1.default(`Selectbox-content text-base`) }, value ? value : '\xa0')),
            React.createElement("div", { className: "Selectbox-icon ml-auto transition" }, arrow ? (arrow) : (React.createElement("div", { className: classnames_1.default('Selectbox-arrow  w-3 h-3 border-black  border-b-2 border-r-2 transition', open ? 'rotate-225' : 'rotate-45', material && 'mr-1') })))),
        React.createElement(react_pose_1.PoseGroup, { preEnterPose: "preEnter" }, open && (React.createElement(AnimatedList, { className: classnames_1.default(`Selectbox-list w-full absolute pin-b pin-l overflow-y-auto z-10`), key: "list", ref: listContainer, role: "listbox" }, options.map((option, index) => {
            let isActive = false;
            if (multiple && selection) {
                // @ts-ignore
                isActive = selection.includes(option.value);
            }
            else {
                isActive = selection === option.value;
            }
            const ListItem = index < 6 ? AnimatedListItem : 'button';
            return (React.createElement(React.Fragment, null,
                React.createElement(ListItem, { "aria-selected": isActive, className: classnames_1.default(`Selectbox-list-item flex items-center w-full px-4 focus:outline-none`, isActive && 'active', index === selIndex && 'selecting'), key: option.label, onClick: () => handleSelect(option), role: "option" },
                    option.iconStart && option.iconStart,
                    multiple && (React.createElement("div", { className: classnames_1.default('Selectbox-list-item-checkbox relative mr-2') })),
                    React.createElement("span", { className: classnames_1.default(`Selectbox-list-item-label`) }, option.label),
                    option.iconEnd && option.iconEnd)));
        })))),
        React.createElement(react_pose_1.PoseGroup, { preEnterPose: "preEnter" }, multiple && open && (React.createElement(AnimatedList, { className: classnames_1.default('Selectbox-list-submitWrapper w-full absolute pin-b pin-l z-1'), key: "selectbox-submit-wrapper" },
            React.createElement("button", { className: classnames_1.default(`Selectbox-list-submit absolute pin-b pin-l flex items-center justify-center w-full px-4 focus:outline-none`), key: "selectbox-submit", onClick: toogleOpen }, multipleSave))))));
});
exports.Selectbox.defaultProps = {
    label: '',
    multipleSave: 'Übernehmen'
};
